proc InitNastranRead { filename } {
    global NastranPriv NastranPrivMat NastranPrivProp NastranPrivElemProp NastranPrivUnimplemented
    if { [info exists NastranPrivMat ] } {
	unset NastranPrivMat
    }
   
    if { [info exists NastranPrivProp ] } {
	unset NastranPrivProp
    }

    if { [info exists NastranPrivUnimplemented ] } {
	unset NastranPrivUnimplemented
    }
    
    if { [info exists NastranPrivElemProp ] } {
	unset NastranPrivElemProp
    }

    set NastranPrivMat(Names) ""
    set NastranPrivMat(Fields,MAT) "Table young  mo_shear poisson density temp_ref \
	    tension compression shear damping expansion conductivity spec_heat free_conv \
	heatgen nontype plasticity harden yield_func initial friction stress-strain"
    set NastranPrivMat(Defaults,MAT) "0 0 0 0 0 0 void void void 0 0 0 0 0 0 0 0 0 0 0 \
	    0 0"

    set NastranPrivProp(Fields,PBAR) "Table PROPERTY win_info Area Moments_of_Inertia I1 I2 I12 Torsional_Constant \
	Y_Shear_Area Z_Shear_Area Nonstructural_mass/lentgh Stress_Recovery Values:(Y,Z) Composition_Material"
    set NastranPrivProp(Defaults,PBAR) [list 0 BAR 0 0.0 I 0.0 0.0 0.0 0.0 0.0 0.0 0.0 "2_to_4_Blank=Square" \
	    "#N# 8 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0" AISI_4340_STEEL]
     
    set NastranPrivProp(Fields,PBEAM) "Table PROPERTY win_info Area Moments_of_Inertia I1 I2 I12 Torsional_Constant \
	Y_Shear_Area Z_Shear_Area Nonstructural_mass/lentgh Y_Neutral_Axis_Offset Stress_Recovery  \
	Z_Neutral_Axis_Offset Values:(Y,Z) Composition_Material"
    set NastranPrivProp(Defaults,PBEAM) [list 0 BAR 0 0.0 I 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 "2_to_4_Blank=Square" \
	    "#N# 8 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0" AISI_4340_STEEL]   
 
    set NastranPrivProp(Fields,PTUBE) "Table PROPERTY Outside_diameter_of_tube Solid_Circular_Rod Thickness_of_tube \
	Nonstructural_mass/length Composition_Material"
    set NastranPrivProp(Defaults,PTUBE) "0 TUBE 0.0 0 0.0 0.0 AISI_4340_STEEL"

    set NastranPrivProp(Fields,PSHELL) "Table PROPERTY Thicknesses,Tagv_or_T1 \
	T2_or_Blank T3_or_Blank T4_or_Blank Nonstructural_mass/area Stress_Recovery \
	Top_Fiber Bottom_Fiber Bend_Stiffness,_12I/T^3 TShear/Mem_Thickness,_ts/t \
	Bending Transverse_Shear Memb-Bend_Coupling Composition_Material" 
    
    set NastranPrivProp(Defaults,PSHELL) "0 PLATE 0.0 0.0 0.0 0.0 0.0 Default=T/2 0.0 \
	0.0 0.0 0.0 Plate_Material Plate_Material Plate_Material AISI_4340_STEEL"

    set NastranPrivProp(Fields,PSOLID) "Table PROPERTY Coord_System Composition_Material"
    set NastranPrivProp(Defaults,PSOLID) "0 TETRAHEDRON ELEMENT AISI_4340_STEEL"

    set NastranPrivProp(Names) ""
    
    set fin [open $filename r]
#   NastranReadExecutiveControl $fin
   NastranReadCaseControl $fin

    #to create conditions inside the first interval data
    GiD_IntervalData set 1
    #set NastranPriv(NotProcessEntitiesInGiD) 0
    #to create also nodes and elements from tcl
}

#fail==0 -> Ok, fail==1 -> Error, fail==2 User stop
proc EndNastranRead { fail } {
    global NastranPrivMat NastranPrivProp NastranPrivElemProp NastranPrivUnimplemented

    foreach i [ array names NastranPrivUnimplemented ] {
	WarnWinText [_ "Warning: Ignored  %s %s entities" $NastranPrivUnimplemented($i) $i]
    }

    #::GidUtils::DisableGraphics
    #::GidUtils::DisableWarnLine
    if { $fail != "2" } {
	#assign element properties
	foreach prop $NastranPrivProp(Names) {
	    if { [info exists NastranPrivElemProp($prop)] } {
		set NastranPrivElemProp($prop) [CompactNumberList $NastranPrivElemProp($prop)]
		GiD_AssignData material $prop elements $NastranPrivElemProp($prop)
		
	    }
	}
    }

    if { [info exists NastranPrivMat ] } {
	unset NastranPrivMat
    }
    if { [info exists NastranPrivProp ] } {
	unset NastranPrivProp
    }

    if { [info exists NastranPrivElemProp ] } {
	unset NastranPrivElemProp
    }
    
    if { [info exists NastranPrivUnimplemented ] } {
	unset NastranPrivUnimplemented
    }

    #::GidUtils::EnableWarnLin
    #::GidUtils::EnableGraphics
    update
}
proc NastranReadExecutiveControl  { fin } {
    global NastranPrivUnimplemented
    set data ""
    while  { [lindex $data 0] != "CEND" } {  
	set data [gets $fin]
	set data [list [lindex $data 0] [lrange $data 1 end]]
	set statment [lindex $data 0]
	switch $statment {
	    ID {
		GiD_AccessValue set gendata ID [split [lrange $data 1 end]]
	    }
	    TIME {
		GiD_AccessValue set gendata TIME [split [lrange $data 1 end]]
	    }
	    SOL - SOLUTION {
		set nastrannames [list [list "STEADY STATE HEAT TRANSFER" 101] [list "LINEAR STATIC" 101]\
		[list "MODAL" 103]  [list "LINEAR BUCKLING" 105] [list  "NONLINEAR STATIC" 106] \
		[list "FREQUENCY RESPONSE" 111] [list "LINEAR TRANSIENT RESPONSE" 112] \
		[list "NONLINEAR BUCKLING" 180] [list  "PRESTRESS STATIC" 181] \
		[list "LINEAR PRESTRESS MODAL" 182] [list "NONLINEAR TRANSIENT RESPONSE" 129] ]
		
		set gidnames [list STEADY_STATE_HEAT_TRANSFER STATIC  MODES BUCKLING NONLINEAR_STATIC \
		DIRECT_FREQUENCY_RESPONSE DIRECT_TRANSIENT_RESPONSE NONLINEAR_BUCKLING \
		PRESTRESS_STATIC  PRESTRESS_MODES NONLINEAR_TRANSIENT]
		#In two previous lists the order of the elements is very important
		if { [string is integer [lindex $data 1]] } {
		    for { set ii 0 } { $i <[llength $nastrannames]} { incr i } {
		        if { [lindex [lindex $nastrannames $ii] 1] == [lindex $data 1]} { 
		            GiD_AccessValue set gendata [lindex $gidnames $ii] 1  
		        } else {
		            GiD_AccessValue set gendata [lindex $gidnames $ii] 0  
		        }
		    }
		} else {
		     for { set ii 0 } { $i <[llength $nastrannames]} { incr i } {
		        if { [lindex [lindex $nastrannames $ii] 0] == "[lindex $data 1]"} { 
		            GiD_AccessValue set gendata [lindex $gidnames $ii] 1  
		        } else {
		            GiD_AccessValue set gendata [lindex $gidnames $ii] 0  
		        }
		    }
		}
	    }
	    DIAG {
		GiD_AccessValue set gendata Diagnostics [split [lrange $data 1 end]]
	    }
	    ANALISYS {
		GiD_AccessValue set gendata Rigid_Format_approach [split [lrange $data 1 end]]
		if { [split [lrange $data 1 end]] == "HEAT" &&  [GiD_AccessValue get gendata STATIC] == 1 } {
		    GiD_AccessValue set gendata STATIC 0
		    GiD_AccessValue set gendata STEADY_STATE_HEAT_TRANSFER 1
		}
	    }
	    "$" {
		#NASTRAN comment line
	    }
	    default {
		WarnWinText "Executive Control: $data not imported"
	    }
	}
    }
}

proc NastranReadCaseControl { fin } {
    
    set GiDoutputrequests [list Displacement Applied_Load Constraint_Force Velocity \
	    Acceleration Element_Stress Element_Force Element_Strain Strain_Energy \
	    Temperature Flux]
    set NASTRANoutputrequests [list DISPLACEMENT LOAD SPCFORCES VELOCITY \
	 ACCELERATION STRESS FORCE ESE STRAIN THERMAL FLUX]
    set GiDcasecontrol [list Title Subtitle Label Lines_per_printed_page Maximum_number_of_output_lines ECHO]
    set NASTRANcasecontrol [list TITLE SUBTITLE LABEL LINE MAXLINES ECHO]
    set data ""
    while {$data != "BEGIN BULK" } {
	set data [gets $fin] 
	if { [regexp {([^=]+)\s*=\s*(.*)} $data {} key value] } {
	    if { [regexp {([^=]+)\s*\(\s*(.*)} $key {} key value] } {
		set key [string trim $key]
		set index [lsearch $NASTRANoutputrequests $key]
		if {  $index != -1 } {
		      GiD_AccessValue set gendata [lindex $GiDoutputrequests $index] 1
		    } else {
		    WarnWinText "Case control: Output $key is not supported"
		}
	    } else { 
		set key [string trim $key]
		set index [lsearch $NASTRANcasecontrol $key]
		if {  $index !=-1} {
		    GiD_AccessValue set gendata [lindex $GiDcasecontrol $index] 1
		}
	    }
	}
    }
}
proc SetMatFieldValue { mat field value } {   
    global NastranPrivMat
    set i [lsearch $NastranPrivMat(Fields,MAT) $field]
    if { $i != "-1" } {
	lset NastranPrivMat($mat) $i "$value"
    } else {
	WarnWinText [_ "Error, field %s not exists in material %s" $field $mat]
    }
}

proc SetPropFieldValue { prop type field value } {   
    global NastranPrivProp
    set i [lsearch $NastranPrivProp(Fields,$type) $field]
    if { $i != "-1" } {
	lset NastranPrivProp($prop) $i $value
    } else {
	WarnWinText [_ "Error, field %s not exists in property %s" $field $prop]
    }
}

#instead "4 8 9 10 11 15" compact list --> "4 8 : 11 15" 
proc AddNumberToNastranPrivElemProp { prop n } {
    global NastranPrivElemProp
    if { ![info exists NastranPrivElemProp($prop)] || [llength $NastranPrivElemProp($prop)] < 2 } {
	lappend NastranPrivElemProp($prop) $n
    } else {
	if { [expr [lindex $NastranPrivElemProp($prop) end]+1] == $n } {
	    if { [lindex $NastranPrivElemProp($prop) end-1] == ":" } {
		lset NastranPrivElemProp($prop) end $n
	    } elseif { [expr [lindex $NastranPrivElemProp($prop) end-1]+1] == [lindex $NastranPrivElemProp($prop) end]} {
		lset NastranPrivElemProp($prop) end :
		lappend NastranPrivElemProp($prop) $n
	    } else {
		lappend NastranPrivElemProp($prop) $n
	    }
	} else {
	    lappend NastranPrivElemProp($prop) $n
	}
    }
}

#items must be positive integer numbers (not 0)
proc CompactNumberList { a } {    
    set ret ""
    set istart 0
    foreach i [lsort -integer $a] {  
	if { !$istart} {
	    set istart $i
	} else {
	    if {$i != [expr $iprev+1]} {
		if { $istart != $iprev } {
		    if { [expr $istart +1] == $iprev } {
		        append ret "$istart $iprev "
		    } else { 
		        append ret "$istart:$iprev "
		    }
		} else {
		    append ret "$istart "
		}            
		set istart $i
	    }        
	}
	set iprev $i
    }
    if { $istart } {
	if { $istart != $iprev } {
	    if { [expr $istart +1] == $iprev } {
		append ret "$istart $iprev"
	    } else { 
		append ret "$istart:$iprev"
	    }
	} else {
	    append ret "$istart"
	}   
    }
    return $ret
}

proc NastranEntity { type data } {
    global NastranPrivMat NastranPrivProp NastranPrivElemProp NastranPrivUnimplemented
    set data [split $data ,]
    switch $type {
	CBAR {
	    set EID [lindex $data 0]
	    set PID [lindex $data 1]
	    set GA [lindex $data 2]
	    set GB [lindex $data 3]
	    set X1 [lindex $data 4] ;#can be X1 or G0
	    set X2 [lindex $data 5]
	    set X3 [lindex $data 6]

	    set PA [lindex $data 8]
	    set PB [lindex $data 9]
	    set W1A [lindex $data 10]
	    set W2A [lindex $data 11]
	    set W2A [lindex $data 12]
	    set W1B [lindex $data 13]
	    set W2B [lindex $data 14]
	    set W2B [lindex $data 15]

	    set prop "PBAR_$PID"
	    append NastranPrivElemProp($prop) "$EID "

	    foreach {Num A1 A2 A3} [.central.s info Mesh Nodes $GA] break
	    foreach {Num B1 B2 B3} [.central.s info Mesh Nodes $GB] break
	    if { [string is integer $X1] } {
		set G0 $X1
		foreach {Num X1 X2 X3} [.central.s info Mesh Nodes $G0] break
	    }
	   
	    set exists [GiD_LocalAxes exists "" rectangular C_XY_X "$A1 $A2 $A3" "$X1 $X2 $X3" "$B1 $B2 $B3"]
	    if { $exists == -1 } {
		set values "GLOBAL"
	    } elseif { $exists == -2 } {
		set values "Auto"
	    } elseif { $exists == -3 } {
		set values "Auto_alt"
	    } elseif { $exists == 0 } {
		GiD_LocalAxes create "LA_$EID" rectangular C_XY_X "$A1 $A2 $A3" "$X1 $X2 $X3" "$B1 $B2 $B3"
	    } else {
		set values [lindex [.central.s info localaxes] [expr $exists-1]]
	    }
	    set cond "Line-Local-Axes"
	    GiD_AssignData condition $cond body_elements $values $EID
	}
	CQUADR - CQUAD4 {
	    set EID [lindex $data 0]
	    set PID [lindex $data 1]
	    #set G1 [lindex $data 2]
	    #set G2 [lindex $data 3]
	    #set G3 [lindex $data 4]
	    #set G4 [lindex $data 5]
	    #...
	    set prop "PSHELL_$PID" ;#PSHELL_OR_PCOMP
	    append NastranPrivElemProp($prop) "$EID "
	    #too slow assign to each element the material, store it an assign only once at end
	}
	CTETRA {
	    set EID [lindex $data 0]
	    set PID [lindex $data 1]
	    # ...conectivities
	    set prop "PSOLID_$PID"
	    append NastranPrivElemProp($prop) "$EID "
	    #lappend NastranPrivElemProp($prop) $EID ;#this is too slow!!!
	    #too slow assign to each element the material, store it an assign only once at end
	}
	CHEXA {
	    set EID [lindex $data 0]
	    set PID [lindex $data 1]
	    set prop "PSLOID_$PID"
	    append NastranPrivElemProp($prop) "$EID"
	    #lappend NastranPrivElemProp($prop) $EID ;#this is too slow!!!
	    #too slow assign to each element the material, store it an assign only once at end
	}
	CTRIAR - CTRIA3 {
	    set EID [lindex $data 0]
	    set PID [lindex $data 1]
	    #set G1 [lindex $data 2]
	    #set G2 [lindex $data 3]
	    #set G3 [lindex $data 4]
	    #...
	    set prop "PSHELL_$PID" ;#PSHELL_OR_PCOMP
	    append NastranPrivElemProp($prop) "$EID "
	    #too slow assign to each element the material, store it an assign only once at end
	}
	FORCE {
	    set SID [lindex $data 0]
	    set G [lindex $data 1]
	    set CID [lindex $data 2]
	    set F [lindex $data 3]
	    set N1 [lindex $data 4]
	    set N2 [lindex $data 5]
	    set N3 [lindex $data 6]
	    set cond "Point-Force-Load"
	    if { $CID } {
		#must calculate the global normal components ...
		WarnWinText "UNDER CONSTRUCTION: $type not implemented for non global coordinate sytems."
	    }
	    set values "[expr $F*$N1] [expr $F*$N2] [expr $F*$N3]"
		        
	    set prevInterval [GiD_IntervalData set]
	    if { $SID < 1 } {
		WarnWinText "WARNING: $type unexpected value of SID='$SID'"
		set SID [expr $prevInterval-1]
	    }

	    if { [expr $SID+1] != $prevInterval} {
		GiD_IntervalData set [expr $SID+1]
	    }
	    GiD_AssignData condition $cond nodes $values $G
	    if { [expr $SID+1] != $prevInterval } {
		#restore interval data
		GiD_IntervalData set $prevInterval
	    }
	}
	MOMENT {
	    set SID [lindex $data 0]
	    set G [lindex $data 1]
	    set CID [lindex $data 2]
	    set M [lindex $data 3]
	    set N1 [lindex $data 4]
	    set N2 [lindex $data 5]
	    set N3 [lindex $data 6]
	    set cond "Moment"
	    if { $CID } {
		#must calculate the global normal components ...
		WarnWinText "UNDER CONSTRUCTION: $type not implemented for non global coordinate sytems."
	    }
	    set values "[expr $M*$N1] [expr $M*$N2] [expr $M*$N3]"
		        
	    set prevInterval [GiD_IntervalData set]
	    if { $SID < 1 } {
		WarnWinText "WARNING: $type unexpected value of SID='$SID'"
		set SID [expr $prevInterval-1]
	    }

	    if { [expr $SID+1] != $prevInterval} {
		GiD_IntervalData set [expr $SID+1]
	    }
	    GiD_AssignData condition $cond nodes $values $G
	    if { [expr $SID+1] != $prevInterval } {
		#restore interval data
		GiD_IntervalData set $prevInterval
	    }
	}
	GRAV {
	    set SID [lindex $data 0]
	    set CID [lindex $data 1]
	    set G [lindex $data 2]
	    set N1 [lindex $data 3]
	    set N2 [lindex $data 4]
	    set N3 [lindex $data 5]
	    #set TID1 [lindex $data 8]
	    #set TID2 [lindex $data 9]
	    #set TID3 [lindex $data 10]
	    
	    #ignored SID, gravity equal foreach interval at this moment...
	    #not implemented CID !=0 at this moment...
	    GiD_AccessValue set gendata :Consider_Acceleration YES
	    GiD_AccessValue set gendata Modul_Acceleration $G
	    GiD_AccessValue set gendata X-Acceleration_Vector $N1
	    GiD_AccessValue set gendata Y-Acceleration_Vector $N2
	    GiD_AccessValue set gendata Z-Acceleration_Vector $N3
	}
	GRID {
	    #set ID [lindex $data 0]
	    #set CP [lindex $data 1]
	    #set X1 [lindex $data 2]
	    #set Y1 [lindex $data 3]
	    #set Z1 [lindex $data 4]
	    #set CD [lindex $data 5]
	    #set PS [lindex $data 6]
	}
	MAT1 {
	    set MID [lindex $data 0]
	    #use NastranPrivMat($mat) to store materials.
	    set mat "MAT_$MID"
	    if { [lsearch $NastranPrivMat(Names) $mat] != "-1"} {
		set newmat 0
	    } else {
		set newmat 1
		set mattemplate AISI_4340_STEEL
		lappend NastranPrivMat(Names) $mat
		set NastranPrivMat($mat) $NastranPrivMat(Defaults,MAT)
	    }

	    SetMatFieldValue $mat young [lindex $data 1] ;#E
	    SetMatFieldValue $mat  mo_shear [lindex $data 2] ;#G
	    SetMatFieldValue $mat poisson [lindex $data 3] ;#NU
	    SetMatFieldValue $mat density [lindex $data 4] ;#RHO
	    SetMatFieldValue $mat expansion [lindex $data 5] ;#A
	    SetMatFieldValue $mat temp_ref [lindex $data 6] ;#TREF
	    SetMatFieldValue $mat damping [lindex $data 7] ;#GE
	    if { [lindex $data 8] != "" } { 
		SetMatFieldValue $mat tension [lindex $data 8] ;#ST 
	    }
	    if { [lindex $data 9] != "" } {
		SetMatFieldValue $mat compression [lindex $data 9] ;#SC
	    }
	    if { [lindex $data 10] != "" } {
		SetMatFieldValue $mat shear [lindex $data 10] ;#SS
	    }
	    #SetMatFieldValue $mat ?? [lindex $data 11] ;#CS
	    
	    if { $newmat } {
		GiD_CreateData create material $mattemplate $mat $NastranPrivMat($mat)
	    } else {
		GiD_ModifyData materials $mat $NastranPrivMat($mat)
	    }
	}
	MATT1 {
	     set MID [lindex $data 0]
	    #use NastranPrivMat($mat) to store materials.
	    set mat "MAT_$MID"
	    if { [lsearch $NastranPrivMat(Names) $mat] != "-1"} {
		set newmat 0
	    } else {
		set newmat 1
		set mattemplate AISI_4340_STEEL
		lappend NastranPrivMat(Names) $mat
		set NastranPrivMat($mat) $NastranPrivMat(Defaults,MAT)
	    }

	    SetMatFieldValue $mat young "Table_[lindex $data 1]" ;#E
	    SetMatFieldValue $mat  mo_shear "Table_[lindex $data 2]";#G
	    SetMatFieldValue $mat poisson "Table_[lindex $data 3]" ;#NU
	    SetMatFieldValue $mat density "Table_[lindex $data 4]" ;#RHO
	    SetMatFieldValue $mat expansion "Table_[lindex $data 5]" ;#A
	    SetMatFieldValue $mat temp_ref "Table_[lindex $data 6]" ;#TREF
	    SetMatFieldValue $mat damping "Table_[lindex $data 7]" ;#GE
	    if { [lindex $data 8] != "" } { 
		SetMatFieldValue $mat tension "Table_[lindex $data 8]" ;#ST 
	    }
	    if { [lindex $data 9] != "" } {
		SetMatFieldValue $mat compression "Table_[lindex $data 9]" ;#SC
	    }
	    if { [lindex $data 10] != "" } {
		SetMatFieldValue $mat shear "Table_[lindex $data 10]" ;#SS
	    }
	    #SetMatFieldValue $mat ?? [lindex $data 11] ;#CS
	    
	    if { $newmat } {
		GiD_CreateData create material $mattemplate $mat $NastranPrivMat($mat)
	    } else {
		GiD_ModifyData materials $mat $NastranPrivMat($mat)
	    }
	}
	MAT4 {
	    set MID [lindex $data 0]
	    #use NastranPrivMat($mat) to store materials.
	    set mat "MAT_$MID"
	    if { [lsearch $NastranPrivMat(Names) $mat] != "-1"} {
		set newmat 0
	    } else {
		set newmat 1
		set mattemplate AISI_4340_STEEL
		lappend NastranPrivMat(Names) $mat
		set NastranPrivMat($mat) $NastranPrivMat(Defaults,MAT)
	    }

	    SetMatFieldValue $mat conductivity [lindex $data 1] ;#K
	    if { [lindex $data 2] != "" } {
		SetMatFieldValue $mat spec_heat [lindex $data 2] ;#CP
	    }
	    if { [lindex $data 3] != "" } { 
		SetMatFieldValue $mat density [lindex $data 3]
	    }
	    if { [lindex $data 4] != "" } { 
		SetMatFieldValue $mat free_conv [lindex $data 4] ;#H
	    }
	    #SetMatFieldValue $mat ?? [lindex $data 5] ;#MU
	    if { [lindex $data 6] != "" } { 
		SetMatFieldValue $mat heatgen [lindex $data 6] ;#HGEN
	    }
	    if { $newmat } {
		GiD_CreateData create material $mattemplate $mat $NastranPrivMat($mat)
	    } else {
		GiD_ModifyData materials $mat $NastranPrivMat($mat)
	    }
	}
	MATT4 {
	    set  MID [lindex $data 0]
	     #use NastranPrivMat($mat) to store materials.
	    set mat "MAT_$MID"
	    if { [lsearch $NastranPrivMat(Names) $mat] != "-1"} {
		set newmat 0
	    } else {
		set newmat 1
		set mattemplate AISI_4340_STEEL
		lappend NastranPrivMat(Names) $mat
		set NastranPrivMat($mat) $NastranPrivMat(Defaults,MAT)
	    }
	    SetMatFieldValue $mat conductivity "Table_[lindex $data 1]" ;#K
	    if { [lindex $data 2] != "" } {
		SetMatFieldValue $mat spec_heat "Table_[lindex $data 2]" ;#CP
	    }
	    if { [lindex $data 3] != "" } { 
		SetMatFieldValue $mat density "Table_[lindex $data 3]" ;#RHO
	    }
	    if { [lindex $data 4] != "" } { 
		SetMatFieldValue $mat free_conv "Table_[lindex $data 4]" ;#H
	    }
	    #SetMatFieldValue $mat ?? [lindex $data 5] ;#MU
	    if { [lindex $data 6] != "" } { 
		SetMatFieldValue $mat heatgen "Table_[lindex $data 6]" ;#HGEN
	    }
	    if { $newmat } {
		GiD_CreateData create material $mattemplate $mat $NastranPrivMat($mat)
	    } else {
		GiD_ModifyData materials $mat $NastranPrivMat($mat)
	    }
	}
	TABLEM2 { 
	    set TID [lindex $data 0]
	}
	PARAM {
	    set N [lindex $data 0]
	    set V [lindex $data 1]
	    switch $N {
		MULTBC - BODYLOAD - GPFORCE - MPCFORCE - INRELF - PELMCHK - PRGPST - 
		AUTOSPC - PRTGPST - BISECT  {
		    if { $V == "YES" ||  $V == "ON" || $V == "1" } {
		        GiD_AccessValue set gendata $N 1
		    } elseif { $V == "NO" || $V == "OFF" || $V == "0" } {
		        GiD_AccessValue set gendata $N 0
		    }
		}
		K6ROT - MAXRATIO - GRDPNT - WTMASS - IRES - SINGOPT - LGDISP {
		    GiD_AccessValue set gendata $N $V
		}
		G {
		   GiD_AccessValue set gendat Overall_Structural_Damping $V
		}
		COUPMASS {
		    GiD_AccessValue set gendat Mass_formulation $V
		}
		W3 {
		    GiD_AccessValue set gendat Frequency_for_System_Damping $V
		}
		W4 {
		    GiD_AccessValue set gendat Frequency_for_Element_Damping $V
		}
		LFREQ {
		    GiD_AccessValue set gendat First_frequency_(Hz) $V
		}
		HFREQ {
		    GiD_AccessValue set gendat Last_frequency_(Hz) $V
		}
		MODACC {
		    GiD_AccessValue set gendat IRES $V

		}
		LANGLE {
		    if { $V == "1" } {
		        GiD_AccessValue set gendat $N GIMBAL
		    } elseif { $V == "2" } {
		        GiD_AccessValue set gendat $N ROTATION
		    }
		}
		NLINSOLACCEL {
		    if { $V == "4" } {
		        GiD_AccessValue set gendat $N 1
		    }
		}
		default {
		    WarnWinText [_ "Warning: Ignored PARAM %s %s" $N $V]
		}
	    }
	}
	PLOAD4 {
	    set SID [lindex $data 0]
	    set EID [lindex $data 1]
	    set P1 [lindex $data 2]
	    set P2 [lindex $data 3]
	    set P3 [lindex $data 4]
	    set P4 [lindex $data 5]
	    set G1 [lindex $data 6]
	    set G34 [lindex $data 7]
	    #set CID [lindex $data 8]
	    set N1 [lindex $data 9]
	    set N2 [lindex $data 10]
	    set N3 [lindex $data 11]

	    if { $N1=="" } { 
		set cond Normal-Surface-Load
		set values $P1 ;#in GiD must be Normal_Inward pressure!!
	    } else {
		set cond Surface-Pressure-Load
		set values [list [expr $P1*$N1] [expr $P1*$N2] [expr $P1*$N3]]
	    }
	  
	    if { $P2 != "" && $P2 != $P1 || $P3 != "" && $P3 != $P1 || \
		    $P4 != "" && $P4 != $P1 } {
		WarnWinText "Warning: PLOAD4 with different nodal pressure values. \
		        Translated as $cond condition with equal noded pressure in element $EID" 
	    }
	  
	    set prevInterval [GiD_IntervalData set]
	    if { $SID < 1 } {
		WarnWinText "WARNING: $type unexpected value of SID='$SID'"
		set SID [expr $prevInterval-1]
	    }
	    if { [expr $SID+1] != $prevInterval} {
		GiD_IntervalData set [expr $SID+1]
	    }                        

	    #GiD element implicit faces
	    #Triangle: 12 23 31
	    #Quadrilateral: 12 23 34 41
	    #Tetrahedra: 123 243 341 421
	    #Hexahedra: 1234 1562 2673 3784 1485 5876
	    set iface 0
	    set a [.central.s info list_entities -more Elements $EID]
	    regexp {Type=([^ ]*)[ ]*Nnode=([^ ]*)} $a dummy elemtype nnode
	    switch $elemtype {
		Triangle {
		    GiD_AssignData condition $cond body_elements $values $EID
		}
		Quadrilateral {
		    GiD_AssignData condition $cond body_elements $values $EID
		}
		Tetrahedra {
		    set globalnodes [lrange $a 12 15]
		    set i [lsearch $globalnodes $G34]
		    if { $i == 0 } {
		        set iface 2
		    } elseif { $i == 1 } {
		        set iface 3
		    } elseif { $i == 2 } {
		        set iface 4
		    } elseif { $i == 3 } {
		        set iface 1
		    } else {
		        WarnWinText [_ "Warning. $s: node %s not found in element %s" $type $G34 $EID]
		    }
		     if { $iface != "0" } {
		        GiD_AssignData condition $cond face_elements $values "$EID $iface"
		    }
		} 
		Hexahedra {
		    set globalnodes [lrange $a 12 17]
		    set i [lsearch $globalnodes $G34]
		    if { $i == 0 } {
		        set iface 2
		    } elseif { $i == 1 } {
		        set iface 3
		    } elseif { $i == 2 } {
		        set iface 4
		    } elseif { $i == 3 } {
		        set iface 5
		    }  elseif { $i == 4 } {
		        set iface 6
		    } elseif { $i == 5 } {
		        set iface 1
		    }
		    else {
		        WarnWinText [_ "Warning. $s: node %s not found in element %s" $type $G34 $EID]
		    }
		    if { $iface != "0" } {
		        GiD_AssignData condition $cond face_elements $values "$EID $iface"
		    }
		}
	    }
	    default {
		WarnWinText [_ "Warning. $s: Unexpected GiD element type %s" $type $elemtype]
	    }

	    if { [expr $SID+1] != $prevInterval } {
		#restore interval data
		GiD_IntervalData set $prevInterval
	    }
	}
	PBAR - PBEAM {
	    set PID [lindex $data 0]
	    set MID [lindex $data 1]
	    
	    set prop "BAR_$PID"
	    if { [lsearch $NastranPrivProp(Names) $prop] != "-1"} {
		set newprop 0
	    } else {
		set newprop 1
		lappend NastranPrivProp(Names) $prop
		set mattemplate Bar
		set NastranPrivProp($prop) $NastranPrivProp(Defaults,PBAR)
	    }
	    SetPropFieldValue $prop PBAR Composition_Material "MAT_$MID"
	    SetPropFieldValue $prop PBAR Area [lindex $data 2]
	    SetPropFieldValue $prop PBAR I1 [lindex $data 3]
	    SetPropFieldValue $prop PBAR I2 [lindex $data 4]
	    SetPropFieldValue $prop PBAR Torsional_Constant [lindex $data 5]
	    SetPropFieldValue $prop PBAR "Nonstructural_mass/lentgh" [lindex $data 6]
	    SetPropFieldValue $prop PBAR "Values:(Y,Z)" "#N# 8 [lrange $data 8 15]"
	    SetPropFieldValue $prop PBAR Y_Shear_Area [lindex $data 16]
	    SetPropFieldValue $prop PBAR Z_Shear_Area [lindex $data 17] 
	    SetPropFieldValue $prop PBAR I12 [lindex $data 18]
	    if { $newprop } {
		GiD_CreateData create material $mattemplate $prop $NastranPrivProp($prop)
	    } else {
		GiD_ModifyData materials $prop $NastranPrivProp($prop)
	    }
	}
	PTUBE {
	    set PID [lindex $data 0]
	    set MID [lindex $data 1]
	     set prop "PTUBE_$PID"
	    if { [lsearch $NastranPrivProp(Names) $prop] != "-1"} {
		set newprop 0
	    } else {
		set newprop 1
		lappend NastranPrivProp(Names) $prop
		set mattemplate Plate
		set NastranPrivProp($prop) $NastranPrivProp(Defaults,PSHELL)
	    }
	    SetPropFieldValue $prop PTUBE Composition_Material "MAT_$MID1"
	    if { [lindex $data 2] != "" } { 
		SetPropFieldValue $prop PTUBE Outside_diameter_of_tube [lindex $data 2]
	    }
	     if { [lindex $data 3] != "" } { 
		if { [lindex $data 3] == 0.0 } {
		    SetPropFieldValue $prop PTUBE Solid_Circular_Rod 1
		    SetPropFieldValue $prop PTUBE Thickness_of_tube [lindex $data 3]
		} else {
		    SetPropFieldValue $prop PTUBE Solid_Circular_Rod 0
		    SetPropFieldValue $prop PTUBE Thickness_of_tube [lindex $data 3]
		}
	    }
	    if { [lindex $data 4] != "" } { 
		SetPropFieldValue $prop PTUBE Nonstructural_mass/length [lindex $data 4]
	    }
	    if { $newprop } {
		GiD_CreateData create material $mattemplate $prop $NastranPrivProp($prop)
	    } else {
		GiD_ModifyData materials $prop $NastranPrivProp($prop)
	    }
	}
	PSHELL {
	    set PID [lindex $data 0]
	    set MID1 [lindex $data 1]
	    set prop "PSHELL_$PID"
	    if { [lsearch $NastranPrivProp(Names) $prop] != "-1"} {
		set newprop 0
	    } else {
		set newprop 1
		lappend NastranPrivProp(Names) $prop
		set mattemplate Plate
		set NastranPrivProp($prop) $NastranPrivProp(Defaults,PSHELL)
	    }

	    SetPropFieldValue $prop PSHELL Composition_Material "MAT_$MID1"
	    if { [lindex $data 2] != ""} { 
		SetPropFieldValue $prop PSHELL Thicknesses,Tagv_or_T1 [lindex $data 2]
		SetPropFieldValue $prop PSHELL T2_or_Blank [lindex $data 2]
		SetPropFieldValue $prop PSHELL T3_or_Blank [lindex $data 2]
		SetPropFieldValue $prop PSHELL T4_or_Blank [lindex $data 2]
	    }
	    if { [lindex $data 7] != ""} { 
		SetPropFieldValue $prop PSHELL Nonstructural_mass/area [lindex $data 7]
	    }
	    if { [lindex $data 8] != ""} { 
		SetPropFieldValue $prop PSHELL Bottom_Fiber [lindex $data 8]
	    }
	    if { [lindex $data 9] != ""} {  
		SetPropFieldValue $prop PSHELL Top_Fiber [lindex $data 9]
	    }
	    if { [lindex $data 4] != ""} {  
		SetPropFieldValue $prop PSHELL Bend_Stiffness,_12I/T^3 [lindex $data 4]
	    }
	    if { [lindex $data 6] != ""} {  
		SetPropFieldValue $prop PSHELL TShear/Mem_Thickness,_ts/t [lindex $data 6]
	    }
	    if { $newprop } {
		GiD_CreateData create material $mattemplate $prop $NastranPrivProp($prop)
	    } else {
		GiD_ModifyData materials $prop $NastranPrivProp($prop)
	    }
	}
	PSOLID {
	    set PID [lindex $data 0]
	    set prop "PSOLID_$PID"
	    if { [lsearch $NastranPrivProp(Names) $prop] != "-1"} {
		set newprop 0
	    } else {
		set newprop 1
		set mattemplate Tetrahedron
		lappend NastranPrivProp(Names) $prop
		set NastranPrivProp($prop) $NastranPrivProp(Defaults,PSOLID)
	    }

	    SetPropFieldValue $prop PSOLID Composition_Material "MAT_[lindex $data 1]";#MID
	    if { [lindex $data 2] == "0" } {  
		;#MCID
		SetPropFieldValue $prop PSOLID Coord_System BASIC
	    } else {
		SetPropFieldValue $prop PSOLID Coord_System ELEMENT
	    }

	    if { $newprop } {
		GiD_CreateData create material $mattemplate $prop $NastranPrivProp($prop)
	    } else {
		GiD_ModifyData materials $prop $NastranPrivProp($prop)
	    }
	}
	SPC {
	    set SID [lindex $data 0]
	    set G1 [lindex $data 1]
	    set C1 [lindex $data 2]
	    set D1 [lindex $data 3]

	    regexp {(1)?(2)?(3)?(4)?(5)?(6)?} $C1 dummy g1 g2 g3 g4 g5 g6   
	    if { $D1 != "" && $D1 != "0" } {        
		set cond Point-Enforced-Displacement
		set values ""
		foreach i {g1 g2 g3 g4 g5 g6} {
		    if { [set $i] == "" } {
		        lappend values 0 
		    } else {
		        lappend values $D1
		    }
		}
	    } else {
		set cond Point-Constraints
		set values ""
		foreach i {g1 g2 g3 g4 g5 g6} {
		    if { [set $i] == "" } {
		        lappend values 0 
		    } else {
		        lappend values 1
		    }
		}
	    }

	    set prevInterval [GiD_IntervalData set]
	    if { $SID < 1 } {
		WarnWinText "WARNING: $type unexpected value of SID='$SID'"
		set SID [expr $prevInterval-1]
	    }
	    if { [expr $SID+1] != $prevInterval} {
		GiD_IntervalData set [expr $SID+1]
	    }                        
	    GiD_AssignData condition $cond nodes $values $G1
	    if { [expr $SID+1] != $prevInterval } {
		#restore interval data
		GiD_IntervalData set $prevInterval
	    }

	}
	default {   
	    if { ![info exists NastranPrivUnimplemented($type)] } {
		set NastranPrivUnimplemented($type) 1
	    } else {
		incr NastranPrivUnimplemented($type)
	    }
	}
    }
}

